@@ -1,4 +1,4 @@
-# -*- coding: iso8859-1 -*-
+# -*- coding: iso-8859-1 -*-
 #
 # Copyright (C) 2003, 2004, 2005 Edgewall Software
 # Copyright (C) 2003, 2004, 2005 Jonas Borgstrm <jonas@edgewall.com>
@@ -16,7 +16,6 @@
 # Author: Jonas Borgstrm <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
 
-from __future__ import generators
 import time
 
 from trac.core import TracError
@@ -102,13 +101,17 @@ class Ticket(object):
             self._old[name] = self.values.get(name)
         elif self._old[name] == value: # Change of field reverted
             del self._old[name]
-        self.values[name] = value.strip()
+        if value:
+            field = [field for field in self.fields if field['name'] == name]
+            if field and field[0].get('type') != 'textarea':
+                value = value.strip()
+        self.values[name] = value
 
     def populate(self, values):
         """Populate the ticket with 'suitable' values from a dictionary"""
         field_names = [f['name'] for f in self.fields]
         for name in [name for name in values.keys() if name in field_names]:
-            self[name] = values.get(name, '').strip()
+            self[name] = values.get(name, '')
 
         # We have to do an extra trick to catch unchecked checkboxes
         for name in [name for name in values.keys() if name[9:] in field_names
@@ -192,9 +195,10 @@ class Ticket(object):
                     and not self._old.has_key('owner'):
                 try:
                     old_comp = Component(self.env, self._old['component'], db)
-                    if old_comp.owner == self.values.get('owner'):
-                        new_comp = Component(self.env, self['component'],
-                                             db)
+                    old_owner = old_comp.owner or ''
+                    current_owner = self.values.get('owner') or ''
+                    if old_owner == current_owner:
+                        new_comp = Component(self.env, self['component'], db)
                         self['owner'] = new_comp.owner
                 except TracError, e:
                     # If the old component has been removed from the database we
@@ -526,7 +530,8 @@ class Milestone(object):
 
     exists = property(fget=lambda self: self._old_name is not None)
     is_completed = property(fget=lambda self: self.completed != 0)
-    is_late = property(fget=lambda self: self.due and self.due < time.time())
+    is_late = property(fget=lambda self: self.due and \
+                                         self.due < time.time() - 86400)
 
     def delete(self, retarget_to=None, author=None, db=None):
         if not db:
