@@ -1,4 +1,4 @@
-# -*- coding: iso8859-1 -*-
+# -*- coding: iso-8859-1 -*-
 #
 # Copyright (C) 2003-2004 Edgewall Software
 # Copyright (C) 2003-2004 Jonas Borgstrm <jonas@edgewall.com>
@@ -14,14 +14,17 @@
 #
 # Author: Jonas Borgstrm <jonas@edgewall.com>
 
-from __future__ import generators
-
+import cgi
+import md5
 import os
+import re
+try:
+    frozenset
+except NameError:
+    from sets import ImmutableSet as frozenset
 import sys
 import time
 import tempfile
-import re
-import md5
 
 TRUE =  ['yes', '1', 1, 'true',  'on',  'aye']
 FALSE = ['no',  '0', 0, 'false', 'off', 'nay']
@@ -38,32 +41,199 @@ def enum(iterable):
         yield idx, item
         idx += 1
 
-def escape(text, quotes=True):
+
+class Markup(str):
+    """Marks a string as being safe for inclusion in XML output without needing
+    to be escaped.
+    
+    Strings are normally automatically escaped when added to the HDF.
+    `Markup`-strings are however an exception. Use with care.
+    
+    (since Trac 0.9.3)
     """
-    Escapes &, <, > and \" so they are safe to include in HTML output. Quotes
-    are only escaped if the `quotes` parameter is `True`; this is only
-    necessary for text that is supposed to be inserted in attributes of HTML
-    tags.
-    """
-    if not text:
-        return ''
-    text = str(text).replace('&', '&amp;') \
-                    .replace('<', '&lt;') \
-                    .replace('>', '&gt;')
-    if quotes:
-        text = text.replace('"', '&#34;')
-    return text
+    def __new__(self, text='', *args):
+        if args:
+            text %= tuple([escape(arg) for arg in args])
+        return str.__new__(self, text)
+
+    def __add__(self, other):
+        return Markup(str(self) + Markup.escape(other))
+
+    def __mul__(self, num):
+        return Markup(str(self) * num)
+
+    def join(self, seq):
+        return Markup(str(self).join([Markup.escape(item) for item in seq]))
+
+    def striptags(self):
+        """Return a copy of the text with all XML/HTML tags removed."""
+        return Markup(re.sub(r'<[^>]*?>', '', self))
+
+    def escape(cls, text, quotes=True):
+        """Create a Markup instance from a string and escape special characters
+        it may contain (<, >, & and ").
+        
+        If the `quotes` parameter is set to `False`, the " character is left as
+        is. Escaping quotes is generally only required for strings that are to
+        be used in attribute values.
+        """
+        if isinstance(text, cls):
+            return text
+        if not text:
+            return cls()
+        text = str(text).replace('&', '&amp;') \
+                        .replace('<', '&lt;') \
+                        .replace('>', '&gt;')
+        if quotes:
+            text = text.replace('"', '&#34;')
+        return cls(text)
+    escape = classmethod(escape)
+
+    def unescape(self):
+        """Reverse-escapes &, <, > and " and returns a `str`."""
+        if not self:
+            return ''
+        return str(self).replace('&#34;', '"') \
+                        .replace('&gt;', '>') \
+                        .replace('&lt;', '<') \
+                        .replace('&amp;', '&')
+
+    def sanitize(self):
+        """Parse the text as HTML and return a cleaned up XHTML representation.
+        
+        This will remove any javascript code or other potentially dangerous
+        elements.
+        
+        If the HTML cannot be parsed, an `HTMLParseError` will be raised by the
+        underlying `HTMLParser` module, which should be handled by the caller of
+        this function.
+        """
+        import htmlentitydefs
+        from HTMLParser import HTMLParser, HTMLParseError
+        from StringIO import StringIO
+
+        buf = StringIO()
+
+        class HTMLSanitizer(HTMLParser):
+            # FIXME: move this out into a top-level class
+            safe_tags = frozenset(['a', 'abbr', 'acronym', 'address', 'area',
+                'b', 'big', 'blockquote', 'br', 'button', 'caption', 'center',
+                'cite', 'code', 'col', 'colgroup', 'dd', 'del', 'dfn', 'dir',
+                'div', 'dl', 'dt', 'em', 'fieldset', 'font', 'form', 'h1', 'h2',
+                'h3', 'h4', 'h5', 'h6', 'hr', 'i', 'img', 'input', 'ins', 'kbd',
+                'label', 'legend', 'li', 'map', 'menu', 'ol', 'optgroup',
+                'option', 'p', 'pre', 'q', 's', 'samp', 'select', 'small',
+                'span', 'strike', 'strong', 'sub', 'sup', 'table', 'tbody',
+                'td', 'textarea', 'tfoot', 'th', 'thead', 'tr', 'tt', 'u', 'ul',
+                'var'])
+            safe_attrs = frozenset(['abbr', 'accept', 'accept-charset',
+                'accesskey', 'action', 'align', 'alt', 'axis', 'border',
+                'cellpadding', 'cellspacing', 'char', 'charoff', 'charset',
+                'checked', 'cite', 'class', 'clear', 'cols', 'colspan', 'color',
+                'compact', 'coords', 'datetime', 'dir', 'disabled', 'enctype',
+                'for', 'frame', 'headers', 'height', 'href', 'hreflang',
+                'hspace', 'id', 'ismap', 'label', 'lang', 'longdesc',
+                'maxlength', 'media', 'method', 'multiple', 'name', 'nohref',
+                'noshade', 'nowrap', 'prompt', 'readonly', 'rel', 'rev', 'rows',
+                'rowspan', 'rules', 'scope', 'selected', 'shape', 'size',
+                'span', 'src', 'start', 'style', 'summary', 'tabindex',
+                'target', 'title', 'type', 'usemap', 'valign', 'value',
+                'vspace', 'width'])
+            uri_attrs = frozenset(['action', 'background', 'dynsrc', 'href',
+                                  'lowsrc', 'src'])
+            safe_schemes = frozenset(['file', 'ftp', 'http', 'https', 'mailto',
+                                      None])
+            empty_tags = frozenset(['br', 'hr', 'img', 'input'])
+            waiting_for = None
+
+            def handle_starttag(self, tag, attrs):
+                if self.waiting_for:
+                    return
+                if tag not in self.safe_tags:
+                    self.waiting_for = tag
+                    return
+                buf.write('<' + tag)
+
+                def _get_scheme(text):
+                    if ':' not in text:
+                        return None
+                    chars = [char for char in text.split(':', 1)[0]
+                             if char.isalnum()]
+                    return ''.join(chars).lower()
+
+                for attrname, attrval in attrs:
+                    if attrname not in self.safe_attrs:
+                        continue
+                    elif attrname in self.uri_attrs:
+                        # Don't allow URI schemes such as "javascript:"
+                        if _get_scheme(attrval) not in self.safe_schemes:
+                            continue
+                    elif attrname == 'style':
+                        # Remove dangerous CSS declarations from inline styles
+                        decls = []
+                        for decl in filter(None, attrval.split(';')):
+                            is_evil = False
+                            if 'expression' in decl:
+                                is_evil = True
+                            for m in re.finditer(r'url\s*\(([^)]+)', decl):
+                                if _get_scheme(m.group(1)) not in self.safe_schemes:
+                                    is_evil = True
+                                    break
+                            if not is_evil:
+                                decls.append(decl.strip())
+                        if not decls:
+                            continue
+                        attrval = '; '.join(decls)
+                    buf.write(' ' + attrname + '="' + escape(attrval) + '"')
+
+                if tag in self.empty_tags:
+                    buf.write(' />')
+                else:
+                    buf.write('>')
+
+            def handle_entityref(self, name):
+                if not self.waiting_for:
+                    if name not in ('amp', 'lt', 'gt', 'quot'):
+                        codepoint = htmlentitydefs.name2codepoint[name]
+                        buf.write(unichr(codepoint).encode('utf-8'))
+                    else:
+                        buf.write('&%s;' % name)
+
+            def handle_data(self, data):
+                if not self.waiting_for:
+                    buf.write(escape(data, quotes=False))
+
+            def handle_endtag(self, tag):
+                if self.waiting_for:
+                    if self.waiting_for == tag:
+                        self.waiting_for = None
+                    return
+                if tag not in self.empty_tags:
+                    buf.write('</' + tag + '>')
+
+        # Translate any character or entity references to the corresponding
+        # UTF-8 characters
+        def _ref2utf8(match):
+            ref = match.group(1)
+            if ref.startswith('x'):
+                ref = int(ref[1:], 16)
+            else:
+                ref = int(ref, 10)
+            return unichr(int(ref)).encode('utf-8')
+        text = re.sub(r'&#((?:\d+)|(?:[xX][0-9a-fA-F]+));?', _ref2utf8, self)
+
+        sanitizer = HTMLSanitizer()
+        sanitizer.feed(text)
+        return Markup(buf.getvalue())
+
+
+escape = Markup.escape
 
 def unescape(text):
-    """
-    Reverse-escapes &, <, > and \".
-    """
-    if not text:
-        return ''
-    return str(text).replace('&#34;', '"') \
-                    .replace('&gt;', '>') \
-                    .replace('&lt;', '<') \
-                    .replace('&amp;', '&') 
+    """Reverse-escapes &, <, > and \"."""
+    if not isinstance(text, Markup):
+        return text
+    return text.unescape()
 
 def to_utf8(text, charset='iso-8859-15'):
     """Convert a string to UTF-8, assuming the encoding is either UTF-8, ISO
@@ -81,13 +251,6 @@ def to_utf8(text, charset='iso-8859-15'):
             u = unicode(text, 'iso-8859-15')
         return u.encode('utf-8')
 
-def sql_escape(text):
-    """
-    Escapes the given string so that it can be safely used in an SQL
-    statement
-    """
-    return text.replace("'", "''").replace("\\", "\\\\")
-
 def shorten_line(text, maxlen = 75):
     if not text:
         return ''
