@@ -1,4 +1,4 @@
-# -*- coding: iso8859-1 -*-
+# -*- coding: iso-8859-1 -*-
 #
 # Copyright (C) 2005 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
@@ -14,8 +14,6 @@
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
-from __future__ import generators
-
 from trac.util import TracError
 from trac.versioncontrol import Changeset, Node, Repository
 
@@ -185,8 +183,7 @@ class SubversionRepository(Repository):
         self.pool = Pool()
         
         # Remove any trailing slash or else subversion might abort
-        if not os.path.split(path)[1]:
-            path = os.path.split(path)[0]
+        path = os.path.normpath(path).replace('\\', '/')
         self.path = repos.svn_repos_find_root_path(path, self.pool())
         if self.path is None:
             raise TracError, \
@@ -466,15 +463,15 @@ class SubversionChangeset(Changeset):
         idx = 0
         copies, deletions = {}, {}
         changes = []
+        revroots = {}
         for path, change in editor.changes.items():
             if not self.authz.has_permission(path):
                 # FIXME: what about base_path?
                 continue
-            if not path.startswith(self.scope[1:]):
+            if not (path+'/').startswith(self.scope[1:]):
                 continue
-            base_path = _scoped_path(self.scope, change.base_path)
             action = ''
-            if not change.path:
+            if not change.path and change.base_path:
                 action = Changeset.DELETE
                 deletions[change.base_path] = idx
             elif change.added:
@@ -485,8 +482,17 @@ class SubversionChangeset(Changeset):
                     action = Changeset.ADD
             else:
                 action = Changeset.EDIT
+                b_path, b_rev = change.base_path, change.base_rev
+                if revroots.has_key(b_rev):
+                    b_root = revroots[b_rev]
+                else:
+                    b_root = fs.revision_root(self.fs_ptr, b_rev, pool())
+                    revroots[b_rev] = b_root
+                change.base_path = fs.node_created_path(b_root, b_path, pool())
+                change.base_rev = fs.node_created_rev(b_root, b_path, pool())
             kind = _kindmap[change.item_kind]
             path = path[len(self.scope) - 1:]
+            base_path = _scoped_path(self.scope, change.base_path)
             changes.append([path, kind, action, base_path, change.base_rev])
             idx += 1
 
@@ -496,6 +502,7 @@ class SubversionChangeset(Changeset):
                 changes[v][2] = Changeset.MOVE
                 moves.append(deletions[k])
         offset = 0
+        moves.sort()
         for i in moves:
             del changes[i - offset]
             offset += 1
