@@ -119,16 +119,15 @@ class ChangesetModule(Component):
                 if chgset.date < stop:
                     message = chgset.message or '--'
                     if format == 'rss':
-                        title = 'Changeset <em>[%s]</em>: %s' \
-                                % (util.escape(chgset.rev),
-                                   util.escape(util.shorten_line(message)))
+                        title = util.Markup('Changeset <em>[%s]</em>: %s',
+                                            chgset.rev,
+                                            util.shorten_line(message))
                         href = self.env.abs_href.changeset(chgset.rev)
-                        message = wiki_to_html(message, self.env, db,
+                        message = wiki_to_html(message, self.env, req, db,
                                                absurls=True)
                     else:
-                        title = 'Changeset <em>[%s]</em> by %s' \
-                                % (util.escape(chgset.rev),
-                                   util.escape(chgset.author))
+                        title = util.Markup('Changeset <em>[%s]</em> by %s',
+                                            chgset.rev, chgset.author)
                         href = self.env.href.changeset(chgset.rev)
                         message = wiki_to_oneliner(message, self.env, db,
                                                    shorten=True)
@@ -143,7 +142,7 @@ class ChangesetModule(Component):
                         message = '<span class="changes">' + ', '.join(files) +\
                                   '</span>: ' + message
                     yield 'changeset', href, title, chgset.date, chgset.author,\
-                          message
+                          util.Markup(message)
                 rev = repos.previous_rev(rev)
 
     # Internal methods
@@ -154,7 +153,7 @@ class ChangesetModule(Component):
         req.hdf['changeset'] = {
             'revision': chgset.rev,
             'time': util.format_datetime(chgset.date),
-            'author': util.escape(chgset.author or 'anonymous'),
+            'author': chgset.author or 'anonymous',
             'message': wiki_to_html(chgset.message or '--', self.env, req,
                                     escape_newlines=True)
         }
@@ -263,8 +262,8 @@ class ChangesetModule(Component):
         """Raw Unified Diff version"""
         req.send_response(200)
         req.send_header('Content-Type', 'text/plain;charset=utf-8')
-        req.send_header('Content-Disposition',
-                        'filename=Changeset%s.diff' % req.args.get('rev'))
+        req.send_header('Content-Disposition', 'inline;'
+                        'filename=Changeset%s.diff' % chgset.rev)
         req.end_headers()
 
         for path, kind, change, base_path, base_rev in chgset.get_changes():
@@ -328,7 +327,7 @@ class ChangesetModule(Component):
         """ZIP archive with all the added and/or modified files."""
         req.send_response(200)
         req.send_header('Content-Type', 'application/zip')
-        req.send_header('Content-Disposition',
+        req.send_header('Content-Disposition', 'attachment;'
                         'filename=Changeset%s.zip' % chgset.rev)
         req.end_headers()
 
@@ -385,15 +384,13 @@ class ChangesetModule(Component):
             return
         authzperm = SubversionAuthorizer(self.env, req.authname)
         db = self.env.get_db_cnx()
-        sql = "SELECT rev,time,author,message " \
-              "FROM revision WHERE %s" % \
-              (query_to_sql(db, query, 'message||author'),)
+        sql, args = query_to_sql(db, query, 'message||author')
         cursor = db.cursor()
-        cursor.execute(sql)
+        cursor.execute("SELECT rev,time,author,message "
+                       "FROM revision WHERE " + sql, args)
         for rev, date, author, log in cursor:
             if not authzperm.has_permission_for_changeset(rev):
                 continue
             yield (self.env.href.changeset(rev),
-                   '[%s]: %s' % (rev, util.escape(util.shorten_line(log))),
-                   date, author,
-                   util.escape(shorten_result(log, query.split())))
+                   '[%s]: %s' % (rev, util.shorten_line(log)),
+                   date, author, shorten_result(log, query.split()))
